% Fisher.m
% Program for "Expectations and the neutrality of interest rates"
% Makes 
% Figure 2 fisher.eps, 
% Figure 4 fisher_transitory.eps, 
% Figure 5 fisher2ct.eps 
% Figure 6 standard_adaptive.eps
%  
% simulates simple models 

clear all
close all

sk = 1; 
r = 0;  
i0 = 0; 
pi0 = i0-r; 
i1 = i0+1; 
rho = 0.99; 

tim = (0:10)';
it = i0*ones(11,1); % program only works for ones, pi_1 is hard wired to AR(1) with eta = 1
it(2:end)=i1; 

pitflex = 0*tim+pi0;
pitflex(3:end)=it(2)-r; 

pitad = 0*tim+pi0;
for t = 2:10;
    pitad(t) = (1+sk)*pitad(t-1) - sk*(it(t)-r);
end

pitftpl = 0*tim+pi0;
pitftpl(2) = rho/(1+sk)+pi0;
for t = 3:10;
    pitftpl(t) = 1/(1+sk)*pitftpl(t-1) + sk/(1+sk)*(it(t-1)-r);
end


fig = figure;
set(fig,'Position',[ 441   377   560   0.6*420]);
hold on
plot(tim,it+0.02,'-bv','linewidth',2)
plot(tim,pitflex,'-m','linewidth',2)
%plot(tim,pitad,'--rv','linewidth',2)
plot(tim,pitftpl,'-rv','linewidth',2)
plot(tim,0*tim,'-k');

axis([0 5 -0.2 1.2])
xticks([0 1 2 3 4 5]);
text(1,1.1,'Interest rate','color','b','fontsize',16);
%text(1.5,-0.6,'Adaptive','color','r','fontsize',16);
text(2.5,0.75,'Sticky price','color','r','fontsize',16)
text(1.4,0.3,'Flexible or Lucas','color','m','fontsize',16)

xlabel('Time');
ylabel('Percent');
print -depsc2 fisher.eps;
print -dpng fisher.png;
% Figure 2    

% transitory
sk = 1; 
rho = 0.99; 
eta = 0.7; 

tim = (0:10)';
it = zeros(11,1); % program only works for ones, pi_1 is hard wired to AR(1) with eta = 1
it(2)=1;
for indx=3:11;
    it(indx) = eta*it(indx-1); 
end; 

pitflex = 0*tim;
pitflex(3:end)=it(2:end-1); 

pitftpl = 0*tim;
pitftpl(2) = rho/(1+sk)*(1-rho)/(1-rho*eta);
for indx = 3:10;
    pitftpl(indx) = 1/(1+sk)*pitftpl(indx-1) + sk/(1+sk)*it(indx-1);
end


fig = figure;
set(fig,'Position',[ 441   377   560   0.6*420]);
hold on
plot(tim,it,'-bv','linewidth',2)
plot(tim,pitflex,'-m','linewidth',2)
%plot(tim,pitad,'--rv','linewidth',2)
plot(tim,pitftpl,'-rv','linewidth',2)
plot(tim,0*tim,'-k');

axis([0 8 -0.2 1.2])
xticks([0 1 2 3 4 5 6 7 8]);
text(0.5,1.1,'Interest rate','color','b','fontsize',16);
%text(1.5,-0.6,'Adaptive','color','r','fontsize',16);
text(4.5,0.6,'Sticky price','color','r','fontsize',16)
text(2.3,1,'Flexible price','color','m','fontsize',16)

xlabel('Time');
ylabel('Percent');
print -depsc2 fisher_transitory.eps;
print -dpng fisher_transitory.png;
% Figure 4     


% simulation that appears to give a negative value
% This is currently not used, I used the continuous time version instead
if 0 ;
    sk = 0.1;
    
    y1 = 1;
    w1 = -0.6;
    z1 = -0.2;
    etay = 0.7;
    etaw = 0.4;
    etaz = 1;
    it = y1*etay.^(tim-1)+w1*etaw.^(tim-1)+z1*etaz.^(tim-1);
    it(1) = 0;
    pit = (((1-rho)/(1+sk)*rho/(1-rho*etay)+sk/(1-etay*(1+sk)))./(1+sk).^(tim-1) - ....
        sk/(1-etay*(1+sk))*etay.^(tim-1))*y1 +...
        (((1-rho)/(1+sk)*rho/(1-rho*etaw)+sk/(1-etaw*(1+sk)))./(1+sk).^(tim-1) - ....
        sk/(1-etaw*(1+sk))*etaw.^(tim-1))*w1 + ...
        (((1-rho)/(1+sk)*rho/(1-rho*etaz)+sk/(1-etaz*(1+sk)))./(1+sk).^(tim-1) - ....
        sk/(1-etaz*(1+sk))*etaz.^(tim-1))*z1 ;
    pit(1)=0;
    
    %xt = -0.5*(it(2:end-1) - pit(3:end));
    %xt = [0; xt; 0];
    
    fig=figure;
    set(fig,'Position',[ 441   377   560   0.6*420]);
    hold on
    plot(tim,it+0.02,'-bv','linewidth',2)
    plot(tim,pit,'-rv','linewidth',2)
    %plot(tim,xt,'--k','linewidth',2)
    plot(tim,0*tim,'-k');
    
    text(3,-0.15,'Inflation','color','r','fontsize',16);
    text(3.5,0.2,'Interest rate','color','b','fontsize',16);
    
    xlabel('Time')
    ylabel('Percent')
    print -depsc2 fisher2.eps;
    print -dpng fisher2.png;
end;

% simulation that appears to give a negative value
%continuous time version . 

sk = 0.1;
r = 1-rho;
y1 = 1;
w1 = -0.6; 
z1 = -0.2; 
etay = 0.35; 
etaw = 0.9; 
etaz = 0; 
timc = (0:0.1:10)';

itc = y1*exp(-etay*timc) + w1*exp(-etaw*timc) + z1*exp(-etaz*timc); 

pit = y1*((r+etay)*sk*exp(-etay*timc) - (r+sk)*etay*exp(-sk*timc))/...
        ((sk-etay)*(r+etay));
pit = pit+... 
      w1*((r+etaw)*sk*exp(-etaw*timc) - (r+sk)*etaw*exp(-sk*timc))/...
        ((sk-etaw)*(r+etaw));
pit = pit+... 
      z1*((r+etaz)*sk*exp(-etaz*timc) - (r+sk)*etaz*exp(-sk*timc))/((sk-etaz)*(r+etaz));
    
fig=figure; 
set(fig,'Position',[ 441   377   560   0.6*420]);
hold on
plot([(-1:0.1:0)';timc],[0*(-1:0.1:0)';itc],'-b','linewidth',2)
plot([(-1:0.1:0)';timc],[0*(-1:0.1:0)';pit],'-r','linewidth',2)
%plot(tim,xt,'--k','linewidth',2)
plot([(-1:0.1:0)';timc],0*[(-1:0.1:0)';timc],'-k');

text(3,-0.12,'Inflation','color','r','fontsize',16);
text(3.5,0.12,'Interest rate','color','b','fontsize',16);

xlabel('Time')
ylabel('Percent')
axis([-1 10 -.20 .3]);
print -depsc2 fisher2ct.eps;
print -dpng fisher2ct.png
% Figure 5


% Standard adaptive expectations model 

sk = 1; 
phi = 1.5; 
rho = 0.01;
T = 10;
dt = 0.01; 
tim = (0:dt:T)';
pit = -1/(phi-1)*(1-exp(-sk*(phi-1)*tim));
it = -1/(phi-1)*(1-phi*exp(-sk*(phi-1)*tim));
rt = exp(-sk*(phi-1)*tim);
rhotvt = (1-exp(-(rho+sk*(phi-1))*tim))/(rho+sk*(phi-1));
ut = 1+0*tim;
ut(1)=0;

fig = figure;
set(fig,'Position',[ 441   377   560   0.6*420]);
hold on;
plot(tim,ut,'--k','linewidth',2);
plot(tim,pit,'-r','linewidth',2);
plot(tim,it,'-b','linewidth',2);
plot(tim,rhotvt,'-m','linewidth',2); 
plot(tim,0*tim,'-k');
axis([0 5 -2 2]); 
ylabel('Percent');
xlabel('Time');
text(2,-0.6,'Interest rate','color','b','fontsize',16);
text(0.5,-1,'Inflation','color','r','fontsize',16);
text(3,1.3,'Debt e^{-r t} v_t','color','m','fontsize',16)
text(0.2,1.3,'Disturbance u_t','color','k','fontsize',16);
%legend('\pi_t','i_t','\rho^t v_t');
xticks([0 1 2 3 4 5]);
yticks([-2 -1 0 1 2]);
print -depsc2 standard_adaptive.eps
print -dpng standard_adaptive.png
% Figure 6 

% adaptive expectations but fiscal constraint
% This is currently not used, fun but needed to cut.
if 0
    T = 10;
    dt = 0.01;
    tim = (0:dt:T);
    sk=1;
    pit = 0*tim;
    it = 0*tim;
    vt = 0*tim;
    rt = 0*tim;
    rhotvt = 0*tim;
    rho = 0.2;
    T1 = 2; % time of negative rates
    r1 = -1;
    rt(tim <= T1)= r1;
    T2 = 2*T1;
    r2 = -exp(rho*T1)*(r1);
    rt((tim >T1)&(tim<T2))=r2;
    pit(tim <= T1) = -sk*tim(tim <= T1)*r1;
    pit((tim >T1)&(tim<=T2)) = -sk*T1*r1 -sk*(tim((tim >T1)&(tim<=T2))-T1)*r2;
    %pit(tim>T2) = pit(max(find(tim<=T2)));
    pit(tim>T2) = -sk*T1*r1 -sk*(T2-T1)*r2;
    %pit2 = -sk*cumsum(rt*dt); checks
    it = rt+pit;
    rhotvt(tim <= T1) = 1/rho*(1-exp(-rho*tim(tim <= T1)))*(r1);
    rhotvt((tim >T1)&(tim<=T2)) = 1/rho*(1-exp(-rho*T1))*(r1) + ...
        exp(-rho*T1)*1/rho*(1-exp(-rho*(tim((tim >T1)&(tim<=T2))-T1)))*(r2);
    
    fig = figure;
    set(fig,'Position',[ 441   377   560   0.6*420]);
    hold on;
    plot(tim,0*tim,'-k');
    %plot(tim,pit2,'-r');
    plot(tim-dt,it,'--b','linewidth',2);
    plot(tim,rt+dt,'-k','linewidth',2)
    plot(tim,rhotvt,'-m','linewidth',2);
    plot(tim,pit,'-r','linewidth',2);
    axis([0 5 -2 3.5]);
    ylabel('Percent');
    xlabel('Time');
    text(2.5,3,'Interest rate','color','b','fontsize',16);
    text(0.5,1.5,'Inflation','color','r','fontsize',16);
    text(2.1,-0.5,'Real rate','color','k','fontsize',16);
    text(2.7,-1.5,'Debt e^{-r t} v_t','color','m','fontsize',16)
    %legend('\pi_t','i_t','r_t','e^{-\rho t} v_t');
    xticks([0 1 2 3 4 5]);
    yticks([-2 -1 0 1 2 3]);
    print -depsc2 fiscal_adaptive.eps
    print -depsc2 fiscal_adaptive.png
end


