% inflation_persist_program_lagged_Phillips

% Makes phillips_with_lag.eps, Figure 3 of "Expectations and the Neutrality
% of Interest Rates" 

% History: Modified from inflation JEP program 
% A modification of the program used for "inflation past present and
% future"; the latter has more variations.
% This version is modified to allow a lag in inflation. 


clear all 
close all

lw=1.5; % plot linewidth
fnt = 16; % plot font

sig = 1; % NK model parameters
kap = .25;

%bet = 1;  a takes its place,   
thpi = 0.0; %taylor response to phi
thx = 0.0; % tayor response to x
rho = 0.99; % linearization in value of debt 
omeg = 0;% 0.9; long or short term debt 
eta = 0.999;% 0.7; %0.999  % interest rate ar(1) 
a = 0.2; % weight on lagged inflation
l = 0; %AR(1) coefficient on lagged inflation variable
M = 1; % downweights future IS term. 

T = 20; 

tim = (0:T)';
pit = 0*tim;

% calculate and plot response to monetary shock
shock = [0 1]';
thpi = 0; 
thx = 0; 
[xt pit vt qt it rnt pt lam]  = solveit(sig,kap, a, l, M, thpi, thx, eta, rho, omeg,T,shock);
disp('eienvalues')
disp(lam)
disp('is real?')
disp(isreal(lam))
%disp('sigma kappa pi(2)');
%disp([sig kap pit(2)]);
pi1min = pit(2);
sigmin = sig;
kapmin = kap; 
pitmin = pit;
amin = a;

figure('Position',[ 441   377   560   0.6*420]); 

hold on;

if 0; % this code chooses parameters randomly. Good for initial exploration
    
    asig=0.5; % sigma of sig is mu*asig
    % X = e^x. Sigma^2(X) = E(X)^2*[e^(sigma^2(x))-1]
    % e^(sigma^2(x)) = 1+Sigma^2(X)/E(X)^2
    sigsig = (log(1+asig^2/(sig^2+asig^2)))^0.5;
    musig=log(sig)-1/2*sigsig^2;
    % reverse engineered so lognormal has mean sig; and stdev asig
    
    akap=0.5; %
    sigkap = (log(1+akap^2/(kap^2+akap^2)))^0.5;
    mukap=log(kap)-1/2*sigkap^2;
    
    rng(1);
    simlength = 5000;
    asim = zeros(simlength,1);
    ksim = zeros(simlength,1);
    sigsim = zeros(simlength,1);
    
    for indx = 1:simlength;
        if 0
            sigi = exp(musig+ sigsig*randn);
            kapi = exp(mukap+ sigkap*randn);
            ai = rand; %uniform between 0 and 1);
        end
        [xti piti vti qti iti rnti pti lam] = solveit(sigi,kapi, a, l, M, thpi, thx, eta, rho, omeg,T,shock);
        %   disp([sigi kapi piti(2)]);
        if isreal(lam);
            plot(tim, piti, '-r','linewidth',1);
            if piti(2)< pi1min;
                pi1min = piti(2);
                sigmin = sigi;
                kapmin = kapi;
                pitmin = piti;
                amin = ai;
                %          disp('lam min');
                %disp(lam');
            end
            asim(indx)=ai;
            ksim(indx)=kapi;
            sigsim(indx)=sigi;
        end;
    end;
end;

% instead, evaluate on grid. Use lesson that sigma kappa < 0.3 for any a
parv = [a kap sig];
agrid = (0:0.05:1)';
kgrid = [0.01; (0.05:0.05:2)'];
sgrid = [0.01; (0.05:0.05:5)'];
for indxa = 1:size(agrid,1);
    ai = agrid(indxa);
    for indxk = 1:size(kgrid,1);
        kapi = kgrid(indxk);
        for indxs = 1:size(sgrid,1);
            sigi = sgrid(indxs);
            if kapi*sigi < 0.3 ; % previously found all > 0.3 don't work
                [xti piti vti qti iti rnti pti lam] = solveit(sigi,kapi, a, l, M, thpi, thx, eta, rho, omeg,T,shock);
                %   disp([sigi kapi piti(2)]);
                if isreal(lam);
                    plot(tim, piti, '-r','linewidth',1);
                    parv = [parv; [ ai kapi sigi]];
                    if piti(2)< pi1min;
                        pi1min = piti(2);
                        sigmin = sigi;
                        kapmin = kapi;
                        pitmin = piti;
                        amin = ai;
                        % disp('lam min');
                        % disp(lam');
                    end;
                end;
            end;
        end;
    end;
end;
    
plot(tim,it,'-b','linewidth',lw);
%plot(tim, pit, '-rv','linewidth',lw); 
%plot(tim, xt,'--ks','linewidth',lw);

%plot(tim, pitmin, '-m','linewidth',lw); 
disp('minimum parameters sig kap a');
disp([sigmin kapmin amin]);
plot(tim,0*tim,'-k');
xlabel('Time');
ylabel('Percent');
axis([0 5 -inf 1.1]);
xticks([0 1 2 3 4 5]);
%yticks([0 0.2 0.4 0.6 0.8 1]);
text(3,0.7,'Inflation \pi','color','r','fontsize',fnt,'horizontalalignment','r');
%text(1.3,0.3,'Minimum \pi_1','color','m','fontsize',fnt);
%text(2.7, -1.5,'Output x','color','k','fontsize',fnt);
text(1,1.05,'Interest rate i','color','b','fontsize',fnt);

if eta>= 0.9;
    print -depsc2 phillips_with_lag.eps;
    print -dpng phillips_with_lag.png;
elseif eta < 0.9
    print -depsc2 phillips_with_lag_trans.eps;
    print -dpng phillips_with_lag_trans.png;
end

%Figure 3

%******
%Warning: only does monetary policy shock. 

function [xt pit vt qt it rnt pt lam] = solveit(sig,kap, a, l, M,  thpi, thx, eta, rho, omeg,T,shock);

show_results = 0; 

N = 4; % ? 

if omeg>0;
    
    A = [...
        M sig 0 0    0    0 ;
        0 1-a 0 0    0    0 ;
        0 0   1 0    0    0;
        0 1   0 rho -omeg 0 ;
        0 0   0 0    omeg 0 ;
        0 0   0 0     0   1];
    
    
    % x pi q ui us
    
    B = [ ...
        1+sig*thx sig*thpi 0  0 0   sig ;
        -kap      1        -a 0 0   0 ;
        0         1-l      l  0 0   0 ;
        0         0        0  1 -1  0 ;
        thx       thpi     0  0 1   1 ;
        0         0        0  0 0   eta];
    
    
    C = [...
        0 0;
        0 0;
        0 0;
        -1 0;
        0 0;
        0 1];
    
    D = [...
        1  sig 0 ;
        0  1-a 0 ;
        0  0   0 ;
        0  0   0 ;
        0  0   omeg ;
        0  0    0 ];
    
elseif omeg == 0; 
    
    A = [...
        M sig 0 0        0 ;
        0 1-a 0 0        0 ;
        0 0   1 0        0;
        0 1   0 rho      0 ;
        0 0   0 0       1];
    
    
    % x pi q ui us
    
    B = [ ...
        1+sig*thx sig*thpi 0  0    sig ;
        -kap      1        -a 0    0 ;
        0         1-l      l  0    0 ;
        thx       thpi     0  1   1 ;
        0         0        0  0    eta];
    
    
    C = [...
        0 0;
        0 0;
        0 0;
        -1 0;
        0 1];
    
    D = [...
        1  sig  ;
        0  1-a  ;
        0  0    ;
        0  0    ;
        0  0    ];
    
end

% Solve by eigenvalues See Fiscal Theory of the Price Level appendix for
% equations

A1 = inv(A); 
F = A1*B; 
[Q L] = eig(F); 
lam=diag(L);
Q1 = inv(Q); 
if show_results;
    disp('Eigenvalues');
    disp(abs(diag(L)'));
end
% produce Ef, Eb, that select forward and backward
% eigvenvalues. If L>=1 in position 1, 3, 
% produce
% 1 0 0 0 
% 0 0 1 0 ...
% for example

nf = find(abs(diag(L))>=1); % nf is the index of eigenvalues greater than one
if show_results
    disp('number of eigenvalues >=1');
    disp(size(nf,1))
end
if (size(nf,1) < size(D,2)); 
    disp('not enough eigenvalues greater than 1');
end;
Ef = zeros(size(nf,1),size(A,2));
Efstar = zeros(size(A,2),size(A,2));
for indx = 1:size(nf,1);
    Ef(indx,nf(indx))=1; 
    Efstar(nf(indx),nf(indx)) = 1; 
end;

nb = find(abs(diag(L))<1);
Eb = zeros(size(nb,1),size(A,2));
for indx = 1:size(nb,1);
    Eb(indx,nb(indx))=1;
end;

ze = Eb*Q1*A1*(C-D*inv(Ef*Q1*A1*D)*Ef*Q1*A1*C);
% how epsilon shocks map to z.
% in principle the forward z are zero. In practice they are 1E-16 and then
% grow. So I go through the trouble of simulating forward only the nonzero
% z and eigenvalues less than one.

Nb = size(Eb,1); % number of stable z's
Lb = (Eb*L*Eb'); % diagonal with only stable Ls

% impulse response
H = T+1;

%shock = [ 1 0]' for fiscal [ 0 1 ]' for monetary; 

zbt = zeros(H,Nb);
zbt(2,:) = (ze*shock)';
for t = 3:H;
    zbt(t,:) = Lb*zbt(t-1,:)';
end;

% now create the full z with zeros
zt = zeros(H,N);
zt(:,nb) = zbt;

% create original variables
yt = zt*Q';

xt = yt(:,1);
pit = yt(:,2);
vt = yt(:,4);
if omeg> 0;
    qt = yt(:,5);
    ut = yt(:,6);
    
    rnt = zeros(H,1);
    rnt = omeg*qt-[0;qt(1:H-1)];
elseif omeg==0;
    ut = yt(:,5); 
end

it = thpi*pit + thx*xt + ut;
if omeg == 0; 
    qt = -it;
    rnt = zeros(H,1);
    rnt = [0;qt(1:H-1)];
end

%yield is nonlinear so use the right size
qlevelt = qt - log(1-omeg);
yldt = exp(-qlevelt)+omeg-1; 

pt = cumsum(pit);
end

